<?php

namespace VisualComposer\Modules\Hub\Download;

if (!defined('ABSPATH')) {
    header('Status: 403 Forbidden');
    header('HTTP/1.1 403 Forbidden');
    exit;
}

use VisualComposer\Framework\Container;
use VisualComposer\Framework\Illuminate\Support\Module;
use VisualComposer\Helpers\Differ;
use VisualComposer\Helpers\Logger;
use VisualComposer\Helpers\Traits\EventsFilters;

class AddonsUpdater extends Container implements Module
{
    use EventsFilters;

    public function __construct()
    {
        $this->addFilter('vcv:hub:download:bundle vcv:hub:download:bundle:addon/*', 'updateAddons');
    }

    protected function updateAddons($response, $payload, Logger $loggerHelper)
    {
        $bundleJson = isset($payload['archive']) ? $payload['archive'] : false;
        if (vcIsBadResponse($response) || !$bundleJson || is_wp_error($bundleJson)) {
            $this->logErrors($response, $loggerHelper, $bundleJson);

            return ['status' => false];
        }
        $hubHelper = vchelper('HubAddons');
        /** @var Differ $addonsDiffer */
        $hubAddons = $hubHelper->getAddons();

        $addonsDiffer = vchelper('Differ');
        if (!empty($hubAddons)) {
            $addonsDiffer->set($hubAddons);
        }

        $fileHelper = vchelper('File');
        $fileHelper->createDirectory($hubHelper->getAddonPath());
        $addonsDiffer->onUpdate(
            [$hubHelper, 'updateAddon']
        )->set(
            $bundleJson['addons']
        );
        $addons = $addonsDiffer->get();
        $hubHelper->setAddons($addons);

        if (!isset($response['addons']) || !is_array($response['addons'])) {
            $response['addons'] = [];
        }

        $addonsKeys = array_keys($bundleJson['addons']);
        foreach ($addonsKeys as $addonsKey) {
            $addonsData = $addons[ $addonsKey ];
            $addonsData['tag'] = $addonsKey;
            if (isset($addonsData['bundlePath'])) {
                if (!$hubHelper->checkAbsUrl($addonsData['bundlePath'])) {
                    $addonsData['bundlePath'] = $hubHelper->getAddonUrl(
                        $addonsKey . '/' . $addonsData['bundlePath']
                    );
                }
            }
            $response['addons'][] = $addonsData;
        }

        return $response;
    }

    /**
     * @param $response
     * @param \VisualComposer\Helpers\Logger $loggerHelper
     * @param $bundleJson
     */
    protected function logErrors($response, Logger $loggerHelper, $bundleJson)
    {
        $messages = [];
        $messages[] = __('Failed to update addons', 'vcwb') . ' #10080';

        if (is_wp_error($response)) {
            /** @var \WP_Error $response */
            $messages[] = implode('. ', $response->get_error_messages()) . ' #10081';
        } elseif (is_array($response) && isset($response['body'])) {
            // @codingStandardsIgnoreLine
            $resultDetails = @json_decode($response['body'], 1);
            if (is_array($resultDetails) && isset($resultDetails['message'])) {
                $messages[] = $resultDetails['message'] . ' #10082';
            }
        }
        if (is_wp_error($bundleJson)) {
            /** @var \WP_Error $bundleJson */
            $messages[] = implode('. ', $bundleJson->get_error_messages()) . ' #10083';
        } elseif (is_array($bundleJson) && isset($bundleJson['body'])) {
            // @codingStandardsIgnoreLine
            $resultDetails = @json_decode($bundleJson['body'], 1);
            if (is_array($resultDetails) && isset($resultDetails['message'])) {
                $messages[] = $resultDetails['message'] . ' #10084';
            }
        }

        $loggerHelper->log(
            implode('. ', $messages),
            [
                'response' => is_wp_error($response) ? 'wp error' : $response,
                'bundleJson' => is_wp_error($bundleJson) ? 'wp error' : $bundleJson,
            ]
        );
    }
}
